@@ -18,18 +18,32 @@ import unittest
 from libcloud.utils.py3 import httplib
 
 from libcloud.compute.drivers.ec2 import EC2NodeDriver, EC2APSENodeDriver
-from libcloud.compute.drivers.ec2 import NimbusNodeDriver, EucNodeDriver
+from libcloud.compute.drivers.ec2 import EC2USWestNodeDriver
+from libcloud.compute.drivers.ec2 import EC2USWestOregonNodeDriver
+from libcloud.compute.drivers.ec2 import EC2EUNodeDriver
+from libcloud.compute.drivers.ec2 import EC2APSENodeDriver
 from libcloud.compute.drivers.ec2 import EC2APNENodeDriver
+from libcloud.compute.drivers.ec2 import EC2APSESydneyNodeDriver
+from libcloud.compute.drivers.ec2 import EC2SAEastNodeDriver
+from libcloud.compute.drivers.ec2 import NimbusNodeDriver, EucNodeDriver
 from libcloud.compute.drivers.ec2 import IdempotentParamError
+from libcloud.compute.drivers.ec2 import REGION_DETAILS
+from libcloud.utils.py3 import urlparse
 from libcloud.compute.base import Node, NodeImage, NodeSize, NodeLocation
 from libcloud.compute.base import StorageVolume
 
-from libcloud.test import MockHttp, LibcloudTestCase
+from libcloud.test import MockHttpTestCase, LibcloudTestCase
 from libcloud.test.compute import TestCaseMixin
 from libcloud.test.file_fixtures import ComputeFileFixtures
 
 from libcloud.test.secrets import EC2_PARAMS
 
+try:
+    parse_qsl = urlparse.parse_qsl
+except AttributeError:
+    import cgi
+    parse_qsl = cgi.parse_qsl
+
 
 class EC2Tests(LibcloudTestCase, TestCaseMixin):
     image_name = 'ec2-public-images/fedora-8-i386-base-v1.04.manifest.xml'
@@ -163,6 +177,22 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         ret = self.driver.ex_stop_node(node)
         self.assertTrue(ret)
 
+    def test_ex_create_node_with_ex_blockdevicemappings(self):
+        EC2MockHttp.type = 'create_ex_blockdevicemappings'
+
+        image = NodeImage(id='ami-be3adfd7',
+                          name=self.image_name,
+                          driver=self.driver)
+        size = NodeSize('m1.small', 'Small Instance', None, None, None, None,
+                        driver=self.driver)
+        mappings = [
+            {'DeviceName': '/dev/sdb', 'VirtualName': 'ephemeral0'},
+            {'DeviceName': '/dev/sdc', 'VirtualName': 'ephemeral1'}
+        ]
+        node = self.driver.create_node(name='foo', image=image, size=size,
+                                       ex_blockdevicemappings=mappings)
+        self.assertEqual(node.id, 'i-2ba64342')
+
     def test_destroy_node(self):
         node = Node('i-4382922a', None, None, None, None, self.driver)
         ret = self.driver.destroy_node(node)
@@ -175,7 +205,8 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
                   ('ec2_us_west', 'us-west-1'),
                   ('ec2_eu_west', 'eu-west-1'),
                   ('ec2_ap_southeast', 'ap-southeast-1'),
-                  ('ec2_ap_northeast', 'ap-northeast-1')
+                  ('ec2_ap_northeast', 'ap-northeast-1'),
+                  ('ec2_ap_southeast_2', 'ap-southeast-2')
                 ]
         for api_name, region_name in names:
             self.driver.api_name = api_name
@@ -194,14 +225,15 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
             self.assertTrue('m2.4xlarge' in ids)
 
             if region_name == 'us-east-1':
-                self.assertEqual(len(sizes), 13)
+                self.assertEqual(len(sizes), 17)
                 self.assertTrue('cg1.4xlarge' in ids)
                 self.assertTrue('cc1.4xlarge' in ids)
                 self.assertTrue('cc2.8xlarge' in ids)
+                self.assertTrue('cr1.8xlarge' in ids)
             elif region_name == 'eu-west-1':
-                self.assertEqual(len(sizes), 11)
+                self.assertEqual(len(sizes), 13)
             else:
-                self.assertEqual(len(sizes), 10)
+                self.assertEqual(len(sizes), 12)
 
         self.driver.region_name = region_old
 
@@ -213,6 +245,12 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
                     'ec2-public-images/fedora-8-i386-base-v1.04.manifest.xml')
         self.assertEqual(image.id, 'ami-be3adfd7')
 
+    def test_list_images_with_image_ids(self):
+        images = self.driver.list_images(ex_image_ids=['ami-be3adfd7'])
+        self.assertEqual(len(images), 1)
+        self.assertEqual(images[0].name,
+                    'ec2-public-images/fedora-8-i386-base-v1.04.manifest.xml')
+
     def test_ex_list_availability_zones(self):
         availability_zones = self.driver.ex_list_availability_zones()
         availability_zone = availability_zones[0]
@@ -343,8 +381,7 @@ class EC2Tests(LibcloudTestCase, TestCaseMixin):
         self.assertTrue(retValue)
 
 
-class EC2MockHttp(MockHttp):
-
+class EC2MockHttp(MockHttpTestCase):
     fixtures = ComputeFileFixtures('ec2')
 
     def _DescribeInstances(self, method, url, body, headers):
@@ -387,6 +424,20 @@ class EC2MockHttp(MockHttp):
         body = self.fixtures.load('run_instances.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
 
+    def _create_ex_blockdevicemappings_RunInstances(self, method, url, body, headers):
+        parameters = dict(parse_qsl(url))
+        self.assertEqual(parameters['BlockDeviceMapping.1.DeviceName'],
+                         '/dev/sdb')
+        self.assertEqual(parameters['BlockDeviceMapping.1.VirtualName'],
+                         'ephemeral0')
+        self.assertEqual(parameters['BlockDeviceMapping.2.DeviceName'],
+                         '/dev/sdc')
+        self.assertEqual(parameters['BlockDeviceMapping.2.VirtualName'],
+                         'ephemeral1')
+
+        body = self.fixtures.load('run_instances.xml')
+        return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
     def _idempotent_RunInstances(self, method, url, body, headers):
         body = self.fixtures.load('run_instances_idem.xml')
         return (httplib.OK, body, {}, httplib.responses[httplib.OK])
@@ -488,22 +539,6 @@ class EucMockHttp(EC2MockHttp):
         return self._CreateTags(method, url, body, headers)
 
 
-class EC2APSETests(EC2Tests):
-    def setUp(self):
-        EC2APSENodeDriver.connectionCls.conn_classes = (None, EC2MockHttp)
-        EC2MockHttp.use_param = 'Action'
-        EC2MockHttp.type = None
-        self.driver = EC2APSENodeDriver(*EC2_PARAMS)
-
-
-class EC2APNETests(EC2Tests):
-    def setUp(self):
-        EC2APNENodeDriver.connectionCls.conn_classes = (None, EC2MockHttp)
-        EC2MockHttp.use_param = 'Action'
-        EC2MockHttp.type = None
-        self.driver = EC2APNENodeDriver(*EC2_PARAMS)
-
-
 class NimbusTests(EC2Tests):
     def setUp(self):
         NimbusNodeDriver.connectionCls.conn_classes = (None, EC2MockHttp)
